//
//  Database Engine Wrapper
//  Copyright  1994, 1995
//  B. W. Bush
//  All rights reserved
//
//  $Revision: 2.0 $
//  $RCSfile: EXAMPLE.CPP $
//  $Date: 1995/07/14 16:17:34 $


// DEW Example


#include <iostream.h>

#include <string.h>

#include <dew.h>


int main()
{
    // Start the engine.

    TDEWEngine engine;

    // Open a database.

    TDEWDatabase database;

    // Try to create the table "DEW_TEST.DB".

    try {

        // Make first field ten characters long called "Name", the second field a date called "Date", and the third field a number called "Amount".

        TDEWFldDesc fields[3];
        fields[0] = TDEWFldDesc(1, "Name", fldZSTRING, 0, 10);
        fields[1] = TDEWFldDesc(2, "Date", fldDATE);
        fields[2] = TDEWFldDesc(3, "Amount", fldFLOAT);

        // Make "Name" + "Date" the primary key.

        DBIKEY keyFld;
        keyFld[0] = 1;
        keyFld[1] = 2;
        TDEWIdxDesc index(keyFld, 2, NULL, 0, NULL, TRUE);

        // Make the table have three fields and one index.

        TDEWCrTblDesc descriptor("DEW_TEST.DB", 3, fields, 1, &index);

        // Create the table.

        database.CreateTable(descriptor);

        cout << "Table \"DEW_TEST.DB\" creation succeeded." << endl;

    } catch(const TDEWException& exception) {

        // Report an error creating the table.

        cerr << "Table \"DEW_TEST.DB\" creation failed: " << exception << endl;

    }

    cout << endl;

    // Try to put some data in the table "DEW_TEST.DB".

    try {

        // Create a cursor for the table and find its record buffer.

        TDEWCursor cursor(database, "DEW_TEST.DB");
        TDEWRecord& record = cursor.Record();

        // Append the data.

        cursor.InitRecord();
        record.PutField("Name", "Arnold");
        record.PutField("Date", TDate());
        record.PutField("Amount", 0);
        cursor.AppendRecord();

        cursor.InitRecord();
        record.PutField(1, "Bruce");
        record.PutField(2, TDate(31, 10, 1994));
        record.PutField(3, 5.00);
        cursor.AppendRecord();

        cursor.InitRecord();
        record.PutField(1, "Charles");
        record.PutField(2, TDate(31, 10, 1994));
        record.PutBlank(3);
        cursor.AppendRecord();

        cursor.InitRecord();
        record.PutField(1, "Charles");
        record.PutField(2, TDate(1, 11, 1994));
        record.PutField(3, -2.50);
        cursor.AppendRecord();

        cout << "Records successfully added." << endl;

    } catch(const TDEWException& exception) {

        // Report an error appending the data.

        cerr << "Records could not be added: " << exception << endl;

    }

    cout << endl;

    // Try to put some data in the table "DEW_TEST.DB" that violates its primary key.

    try {

        // Create a cursor for the table and find its record buffer.

        TDEWCursor cursor(database, "DEW_TEST.DB");
        TDEWRecord& record = cursor.Record();

        // Append some data that has the same key as data previously added.

        cursor.InitRecord();
        record.PutField("Name", "Arnold");
        record.PutField("Date", TDate());
        record.PutField("Amount", 10);
        try {
            cursor.AppendRecord();
            cerr << "Key violation not detected." << endl;
        } catch (const TDEWIntegrityException& exception) {
            if (exception.Result() == DBIERR_KEYVIOL)
                cout << "Key violation successfully detected." << endl;
            else
                throw;
        }

    } catch(const TDEWException& exception) {

        // Report a failure to catch the key violation.

        cerr << "Key violation not detected." << endl;

    }

    cout << endl;

    // Try to extract the data from the table "DEW_TEST.DB".

    try {

        // Create a cursor for the table and find its record buffer.

        TDEWCursor cursor(database, "DEW_TEST.DB");
        TDEWRecord& record = cursor.Record();

        // Print the attributes (field) names.

        cout << "Reading (Name, Date, Amount) . . . " << endl;

        // Print all of the tuples (records).

        for (cursor.SetToBegin(); cursor.Advance(); ) {
            string name;
            TDate date;
            float amount;
            record.GetField("Name", name);
            record.GetField("Date", date);
            const BOOL blank = record.GetField("Amount", amount);
            #ifdef __BCPLUSPLUS__
                date.SetPrintOption(TDate::Numbers);
            #endif
            cout << "\t(\"" << name << "\", " << date << ", ";
            if (blank)
                cout << "<blank>";
            else
                cout << amount;
            cout << ")" << endl;
        }

        cout << "Data successfully read." << endl;

    } catch(const TDEWException& exception) {

        // Report an error reading the data.

        cerr << "Data could not be read: " << exception << endl;

    }

    cout << endl;

   // Try to search for a key value in "DEW_TEST.DB".

    try {

        // Create a cursor for the table and find its record buffer.

        TDEWCursor cursor(database, "DEW_TEST.DB");
        TDEWRecord& record = cursor.Record();

        // Create a record buffer for the key.

        TDEWKey key(cursor);
        TDEWRecord keyRecord(cursor);
        keyRecord.PutField("Name", "Bruce");
        cursor.ExtractKey(key, keyRecord);

        // Search for the key.

        cout << "Searching for (\"Bruce\", *, *) . . ." << endl;
        cursor.GetRecordForKey(key, 1);
        string name;
        TDate date;
        float amount;
        record.GetField("Name", name);
        record.GetField("Date", date);
        record.GetField("Amount", amount);
        #ifdef __BCPLUSPLUS__
            date.SetPrintOption(TDate::Numbers);
        #endif
        cout << "\t(\"" << name << "\", " << date << ", " << amount << ")" << endl;

        cout << "Search successfully completed." << endl;

    } catch(const TDEWException& exception) {

        // Report a failure of the search.

        cerr << endl << "Key could not be found: " << exception << endl;

    }

    cout << endl;

   // Try to set and retrieve a bookmark in "DEW_TEST.DB".

    try {

        // Create a cursor for the table and find its record buffer.

        TDEWCursor cursor(database, "DEW_TEST.DB");
        TDEWRecord& record = cursor.Record();

        // Create a bookmark.

        TDEWBookMark bookMark(cursor);

        cursor.Advance();
        cursor.Advance();

        string name;
        record.GetField("Name", name);
        cout << "Setting book mark at \"" << name << "\"" << endl;
        cursor.GetBookMark(bookMark);

        // Go to the book mark.

        cursor.SetToBegin();
        cout << "Moving to book mark . . . \"";
        cursor.SetToBookMark(bookMark);
        string name2;
        record.GetField("Name", name2);
        cout << name2 << "\"" << endl;

        cout << "Bookmark successfully found." << endl;

    } catch(const TDEWException& exception) {

        // Report a failure of the search.

        cerr << endl << "Bookmark could not be found: " << exception << endl;

    }

    cout << endl;

    // Try to delete the table "DEW_TEST.DB".

    try {

        // Delete the table.

        database.DeleteTable("DEW_TEST.DB");

        cout << "Table successfully deleted." << endl;

    } catch(const TDEWException& exception) {

        // Report an error deleting the table.

        cerr << "Table could not be deleted: " << exception << endl;

    }

    return 0;
}
